"""Agent package for FEID."""

from .maestro import AgenteMaestroNASA

# Re-export configuration (now in core)
from ..core import ConfigAgente

# Re-export enterprise features for backwards compatibility
from ..enterprise import (
    CorrelationTracker,
    CircuitBreaker,
    CircuitBreakerState,
    RetryStrategy,
    ExponentialBackoffStrategy,
    LinearBackoffStrategy,
    FixedDelayStrategy,
    RateLimiter,
)

# Re-export security for backwards compatibility
from ..security import ValidadorPayload, SandboxSeguridad

# Re-export audit for backwards compatibility  
from ..audit import AuditorEstructurado, GestorPoliticasError, SondeoSalud

# Re-export core for backwards compatibility
from ..core import (
    EventoSchema,
    LifecycleHooks,
    GestorAntiInanicion,
    ColectorMetricas,
    PoliticaCola,
    GestorColaPolitica,
)

# Import PoliticaError (now independent) for backwards compatibility
from feid.audit import PoliticaError

__all__ = [
    "AgenteMaestroNASA",
    "ConfigAgente",
    "CorrelationTracker",
    "CircuitBreaker",
    "CircuitBreakerState",
    "RetryStrategy",
    "ExponentialBackoffStrategy",
    "LinearBackoffStrategy",
    "FixedDelayStrategy",
    "RateLimiter",
    "ValidadorPayload",
    "SandboxSeguridad",
    "AuditorEstructurado",
    "GestorPoliticasError",
    "PoliticaError",
    "SondeoSalud",
    "EventoSchema",
    "LifecycleHooks",
    "GestorAntiInanicion",
    "ColectorMetricas",
    "PoliticaCola",
    "GestorColaPolitica",
]
