from feid.agent import AgenteMaestroNASA


class AgenteDummy(AgenteMaestroNASA):
    def _perception_engine(self):
        return None

    def _technical_work(self, task):
        return f"ok:{task}"


def test_instanciacion_basica():
    agente = AgenteDummy("DUMMY")
    assert agente.name == "DUMMY"
