"""FEID - Framework Experimental para Integracion de Datos.

Framework experimental para la construccion de sistemas multi-agente.
"""

__version__ = "0.1.0"
__author__ = "Leon Alberne Torres Restrepo"
__email__ = "albernetorres@gmail.com"
__github__ = "https://github.com/albernetr"
# Re-export enterprise features
from .enterprise import (
    CorrelationTracker,
    CircuitBreaker,
    CircuitBreakerState,
    RetryStrategy,
    ExponentialBackoffStrategy,
    LinearBackoffStrategy,
    FixedDelayStrategy,
    RateLimiter,
)

# Re-export security components
from .security import ValidadorPayload, SandboxSeguridad

# Re-export audit components
from .audit import AuditorEstructurado, GestorPoliticasError, SondeoSalud

# Re-export core utilities
from .core import (
    Config,
    ConfigAgente,
    EventoSchema,
    LifecycleHooks,
    GestorAntiInanicion,
    ColectorMetricas,
    PoliticaCola,
    GestorColaPolitica,
)

# Re-export agent base
from .agent import AgenteMaestroNASA

__all__ = [
    # Core configuration
    "Config",
    "ConfigAgente",
    # Core agent
    "AgenteMaestroNASA",
    # Enterprise
    "CorrelationTracker",
    "CircuitBreaker",
    "CircuitBreakerState",
    "RetryStrategy",
    "ExponentialBackoffStrategy",
    "LinearBackoffStrategy",
    "FixedDelayStrategy",
    "RateLimiter",
    # Security
    "ValidadorPayload",
    "SandboxSeguridad",
    # Audit
    "AuditorEstructurado",
    "GestorPoliticasError",
    "SondeoSalud",
    # Core
    "EventoSchema",
    "LifecycleHooks",
    "GestorAntiInanicion",
    "ColectorMetricas",
    "PoliticaCola",
    "GestorColaPolitica",
]